<?php

 // THIS FILE IS PART OF WEBMATIC3 (Webmatic cube)
 // www.valarsoft.com

 defined('WM3_cEXEC') or die('Denied');

 class webmatic3
  {
   // Permanent objects
   private $_language,$_skin,$_panel,$_page,$_section;

   // Variables
   private $stage,$section,$action;
   private $authenticate_userID,$authenticate_adminID;
   private $home_languageID,$admin_languageID,$preferences_languageID;
   private $setupOn,$warningOn,$debugOn,$setup_ip;

   // Arrays
   private $_PREFERENCES,$_PREFERENCESS_CONTENTS;

   function __construct($setup_on,$warning_on,$debug_on,$setup_ip)
    {
     global $WM3_cQUERY_AFFECTED;

     $WM3_cQUERY_AFFECTED = 0;

     $this->setupOn   = $setup_on;
     $this->warningOn = $warning_on;
     $this->debugOn   = $debug_on;
	 $this->setup_ip  = $setup_ip;

     // Parse some input
     wm3Util::stripMagicQuotes();
     wm3Util::setIDToInt();
     wm3Util::parseSearchFilter();

     // Autosetup
     if(!$this->setupOn) // Setup its disabled
      {
	   if(file_exists('wm3_config.php')) require_once('wm3_config.php');
	   else                              wm3Util::dieMessage("Setup error","Webmatic needs setup but setup is disabled. Please enable the setup!");
      }
     else
      {
       if(!wm3Setup::configNeeded($this,$this->setupOn)) // No setup needed
        {
         require_once('wm3_config.php');
        }
       else // Setup needed and run it
        {
		 if(!defined('WM3_cCONFIG_STARTING_LANGUAGE')) define('WM3_cCONFIG_STARTING_LANGUAGE',WM3_cLANGUAGE_EN); // Needed by language object when do the setup
		 $this->admin_languageID = wm3Language::writeCookieAdminLanguageID();
		 $this->_language = new wm3Language($this->admin_languageID,true);
         wm3Setup::executeSetup($this,$this->setup_ip);
         // Security warning and debug informations
         if($this->setupOn)   $this->warningSetup();
         if($this->warningOn) $this->warning();
         if($this->debugOn)   $this->debug();	 
         exit; // Setup not complete, can not continue
        }
      }

     // Setup passed and webmatic can start
     $this->checkVariables();
     $this->home_languageID        = wm3Language::writeCookieHomeLanguageID();
     $this->admin_languageID       = wm3Language::writeCookieAdminLanguageID();
     $this->preferences_languageID = wm3Language::writeCookiePreferencesLanguageID();
     $this->setPreferences();
     $this->setPreferencesContents();
     switch($this->getStage())
      {
       // Set correct stage language
       case WM3_cSTAGE_ADMIN: { $this->_language = new wm3Language($this->admin_languageID); break; }
       default:               { $this->_language = new wm3Language($this->home_languageID);  break; }
      }
     $this->_skin     = new wm3Skin($this,
	                                $this->_PREFERENCES['skin'],
                                    array(
                                          'logo'        => $this->_PREFERENCES['logo'],
                                          'logo_width'  => $this->_PREFERENCES['logo_width'],
                                          'logo_height' => $this->_PREFERENCES['logo_height']
                                         ),
                                    array(
                                          $this->_PREFERENCES['banner1ID'],
                                          $this->_PREFERENCES['banner2ID'],
                                          $this->_PREFERENCES['banner3ID'],
                                          $this->_PREFERENCES['banner4ID'],
                                          $this->_PREFERENCES['banner5ID'],
                                          $this->_PREFERENCES['banner6ID'],
                                          $this->_PREFERENCES['banner7ID'],
                                          $this->_PREFERENCES['banner8ID'],
                                          $this->_PREFERENCES['banner9ID']
                                         )
                                   );
     $this->_panel    = new wm3Panel();
     $this->_page     = new wm3Page();
     $this->_section  = new wm3Section();
     $this->authenticate_userID = $this->authenticate_adminID = WM3_cUSER_ANONYMOUS;
    }

   public function exec()
    {
     $this->doAuthentication();
     wm3Session::manageSession($this);
     $this->createContent();
     $this->renderContent();
    }

   private function doAuthentication()
    {
     if($this->stage == WM3_cSTAGE_HOME)            $this->doAuthenticateHome();
     if($this->stage == WM3_cSTAGE_USER)            $this->doAuthenticateUser();
     if($this->stage == WM3_cSTAGE_ADMIN)           $this->doAuthenticateAdmin();
	 if($this->stage == WM3_cSTAGE_CHAT)            $this->doAuthenticateChat();
	 if($this->stage == WM3_cSTAGE_RSS)             $this->doAuthenticateRss();
	 if($this->stage == WM3_cSTAGE_BANNER_EXCHANGE) $this->doAuthenticateBannerExchange();
    }

   private function createContent()
    {
     if($this->stage == WM3_cSTAGE_HOME)           {
     	                                            $this->_page->createHomePage($this);
     	                                            $this->_panel->createPanels($this);
     	                                           }
     if($this->stage == WM3_cSTAGE_USER)            $this->_page->createUserPage($this);
     if($this->stage == WM3_cSTAGE_ADMIN)           $this->_page->createAdminPage($this);
	 if($this->stage == WM3_cSTAGE_CHAT)            $this->_page->createChatPage($this);
	 if($this->stage == WM3_cSTAGE_RSS)             $this->_page->createRssPage($this);
	 if($this->stage == WM3_cSTAGE_BANNER_EXCHANGE) $this->_page->createBannerExchangePage($this);
    }

   private function renderContent()
    {
     // Adjust rows width
     $this->_skin->makeWidth(WM3_cPANEL_LEFT, $this);
     $this->_skin->makeWidth(WM3_cPANEL_RIGHT,$this);

     // Render head
     switch($this->stage)
      {
       case WM3_cSTAGE_ADMIN:           { require($this->_skin->getAdminHead($this)); break; }
       case WM3_cSTAGE_CHAT:            { require($this->_skin->getChatHead($this)); break; }
	   case WM3_cSTAGE_RSS:             { break; }
	   case WM3_cSTAGE_BANNER_EXCHANGE: { break; }
       default:                         { require($this->_skin->getHead($this)); }
      }

     // Render body
     switch($this->stage)
      {
	   case WM3_cSTAGE_RSS:             { break; }
	   case WM3_cSTAGE_BANNER_EXCHANGE: { break; }
       default:                         { require($this->_skin->getBody($this)); }
      }

     // Render foot
     switch($this->stage)
      {
       case WM3_cSTAGE_ADMIN:           { require($this->_skin->getAdminFoot($this)); break; }
	   case WM3_cSTAGE_CHAT:            { require($this->_skin->getChatFoot($this)); break; }
	   case WM3_cSTAGE_RSS:             { break; }
	   case WM3_cSTAGE_BANNER_EXCHANGE: { break; }
       default:                         { require($this->_skin->getFoot($this)); }
      }

     // Security warning and debug informations
	 if($this->stage != WM3_cSTAGE_RSS && $this->stage != WM3_cSTAGE_BANNER_EXCHANGE)
	  {
       if($this->setupOn)   $this->warningSetup();
       if($this->warningOn) $this->warning();
       if($this->debugOn)   $this->debug();
      }
    }

   private function doAuthenticateHome()
    {
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication();
     $this->authenticate_adminID = wm3User::checkAdminAuthentication();
    }

   private function doAuthenticateUser()
    {
     // TODO le variabili (in genere provenienti da campi delle form) vanno quatate ??? Si!, HQuot???
     // User authentication and logout
     if($this->action == WM3_cACTION_LOGOUT) wm3User::logoutUser();
     $this->authenticate_userID = wm3User::setUserCookies();
	 $this->authenticate_adminID = wm3User::checkAdminAuthentication();
    }

   private function doAuthenticateAdmin()
    {
     // TODO le variabili (in genere provenienti da campi delle form) vanno quatate ??? Si!, HQuot???
     // Admin authentication and logout
     if($this->action == WM3_cACTION_LOGOUT) wm3User::logoutAdmin();
     $this->authenticate_adminID = wm3User::setAdminCookies();
	 $this->authenticate_userID  = wm3User::checkUserAuthentication();
    }

   private function doAuthenticateChat()
    {
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication();
     $this->authenticate_adminID = wm3User::checkAdminAuthentication();
    }

   private function doAuthenticateRss()
    {//TODO serve per gli RSS???
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication();
     $this->authenticate_adminID = wm3User::checkAdminAuthentication();
    }

   private function doAuthenticateBannerExchange()
    {//TODO serve per gli RSS???
     // Set some cookies if there are user or admin already authenticate
     $this->authenticate_userID  = wm3User::checkUserAuthentication();
     $this->authenticate_adminID = wm3User::checkAdminAuthentication();
    }

   private function checkVariables()
    {
     $this->stage = $this->section = $this->action = 0;
	 if(isset($_GET ['stage']))   $this->stage = abs(intval($_GET ['stage'])) > WM3_cSTAGE_BANNER_EXCHANGE ? 0 : abs(intval($_GET ['stage'])); 
	 if(isset($_POST['stage']))   $this->stage = abs(intval($_POST['stage'])) > WM3_cSTAGE_BANNER_EXCHANGE ? 0 : abs(intval($_POST['stage'])); 
     if(isset($_GET ['section'])) $this->section = intval($_GET ['section']);
     if(isset($_POST['section'])) $this->section = intval($_POST['section']);
     if(isset($_GET ['action']))  $this->action  = intval($_GET ['action']);
     if(isset($_POST['action']))  $this->action  = intval($_POST['action']);
    }

   public static function getPostValueArray($_FIELD,$quote_db = 0)
    {
     // Get all the post variables value needed and put it into the fields array
	 $_AUX = array();
     for($i = 0; $i < sizeof($_FIELD); $i++)
      {
       if($quote_db) $_AUX[$_FIELD[$i]] = wm3Util::quoteDB(webmatic3::getVariable($_FIELD[$i],'post'));
       else          $_AUX[$_FIELD[$i]] = webmatic3::getVariable($_FIELD[$i],'post');
      }
     return $_AUX;
    }

   public static function getRowValueArray($_FIELD,$row)
    {
     // Get all $row values and put it into the fields array
     for($i = 0; $i < sizeof($_FIELD); $i++)
      {
       if(property_exists($row,$_FIELD[$i])) $_AUX[$_FIELD[$i]] = $row->$_FIELD[$i];
      }
     return $_AUX;
    }

   public function includeJavaScript()
    {
     return "<script language=\"JavaScript\" type=\"text/javascript\">".wm3Util::renderJavaScriptConstants()."</script>
	         <script language=\"JavaScript\" src=\"".WM3_cPATH_JAVASCRIPT."lib.js\"    type=\"text/javascript\"></script>
			 <script language=\"JavaScript\" src=\"".WM3_cPATH_JAVASCRIPT."editor.js\" type=\"text/javascript\"></script>";
    }

   public function setPreferences() { $this->_PREFERENCES = wm3Preferences::loadPreferences(); }
 
   public function setPreferencesContents() { $this->_PREFERENCES_CONTENTS = wm3Preferences::loadPreferencesContents($this->home_languageID); }
 
   public function setHomeLanguage($languageID) { $this->home_languageID = $languageID; }
   
   public function setAdminLanguage($languageID) { $this->admin_languageID = $languageID; }

   public function setPreferencesLanguage($languageID) { $this->preferences_languageID = $languageID; }
 
   public function getPreferences($n)         { return $this->_PREFERENCES[$n]; }
   
   public function getPreferencesContents($n) { return $this->_PREFERENCES_CONTENTS[$n]; }
   
   public function getWord($i)                         { return $this->_language->getWord($i); }
 
   public function getWordMonth($i)                    { $array = $this->_language->getArrayMonth(); return $array[$i-1]; }
   
   public function getWordDayOfTheWeek($i)             { $array = $this->_language->getArrayDayOfTheWeek(); return $array[$i-1]; }
   
   public function getWordZodiacSign($i,$info = false) { $array = $this->_language->getArrayZodiacSign(); if($info) return $array[$i-1][1]; else return $array[$i-1][0]; }
   
   public function getHomeLanguage() { return $this->home_languageID; }
   
   public function getAdminLanguage() { return $this->admin_languageID; }

   public function getPreferencesLanguage() { return $this->preferences_languageID; }
   
   public function getAuthenticateUserID() { return $this->authenticate_userID; }
   
   public function getAuthenticateAdminID() { return $this->authenticate_adminID; }

   public function getAuthenticateAdminUserID()
    {
	 // Return authenticated adminID (or userID if administrator its not logged)
	 if($this->authenticate_adminID) return $this->authenticate_adminID;
	 return $this->authenticate_userID;
	}

   public function getSkinObj() { return $this->_skin; }
   
   public function getPanelObj() { return $this->_panel; }
   
   public function getPageObj() { return $this->_page; }
   
   public function getLanguageObj() { return $this->_language; }

   public function getSectionObj() { return $this->_section; }

   public function getStage() { return $this->stage; }
   
   public function getSection() { return $this->section; }
   
   public function getAction() { return $this->action; }
   
   public function getVariable($name,$method)
    {
     if($method == "get")  if(isset($_GET [$name])) return $_GET [$name];
     if($method == "post") if(isset($_POST[$name])) return $_POST[$name];
     if($method == "both")
      {
       if(isset($_GET [$name])) return $_GET [$name];
       if(isset($_POST[$name])) return $_POST[$name];
      }
   	}

   private function warningSetup()
    {
     if($this->setupOn)
      {
       $str = "<br><table width=\"50%\" border=\"1\" align=\"center\"><tr><td bgcolor=\"#ff2222\"><b>Warning:</b></td></tr>
                <tr><td bgcolor=\"#ffffff\">Setup its on.<br>Its very insicure keep it on, please set it to off from <b>index.php</b>.</td></tr>
               </table>";
       echo $str;
      }
    }

   private function warning()
    {
     if($this->warningOn)
      {
	   // TODO
       $str  = "<br><table width=\"50%\" border=\"1\" align=\"center\"><tr><td colspan=\"3\" bgcolor=\"#ffaa33\"><b>Warning:</b></td></tr>";
       $str .= "<tr><td bgcolor=\"#ffffff\">IMAP extension</td><td bgcolor=\"#ffffff\">Not loaded</td><td bgcolor=\"#ffffff\">Must be loaded</td></tr>";
	   $str .= "<tr><td bgcolor=\"#ffffff\">Upload folders</td><td bgcolor=\"#ffffff\">Write permission not granted</td><td bgcolor=\"#ffffff\">Write permissions must be enabled on folders</td></tr>";
       $str .= "</table>";
       echo $str;
      }
    }
   
   private function debug()
    {
     global $WM3_cQUERY_AFFECTED, $WM3_cCLASSES_LOADED;

     $str = "<br><table width=\"50%\" border=\"1\" align=\"center\"><tr bgcolor=\"#00ee00\"><td colspan=\"5\"><b>Debug information:</b></td></tr>";
     $col="#ffffff";

     // Post variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Post variables (_POST):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_POST,true)."</pre></td>
              </tr>";

     // Get variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Get variables (_GET):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_GET,true)."</pre></td>
              </tr>";

     // Cookies variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Cookies variables (_COOKIE):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_COOKIE,true)."</pre></td>
              </tr>";

     // Session variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Session variables (_SESSION):</b></td></tr>";
	 if(isset($_SESSION))
      $str .= "<tr bgcolor=\"$col\">
                <td colspan=\"2\"><pre>".print_r($_SESSION,true)."</pre></td>
               </tr>";
	 else $str .= "<tr bgcolor=\"$col\"><td colspan=\"2\"><pre>".print_r(array(),true)."</pre></td></tr>";

     // Uploaded files
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Uploaded files (_FILES):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_FILES,true)."</pre></td>
              </tr>";

     // Server variables
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Server variables (_SERVER):</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\"><pre>".print_r($_SERVER,true)."</pre></td>
              </tr>";              

     // Classes loaded
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Classes loaded:</b></td></tr>";
     $str .= "<tr bgcolor=\"$col\">
               <td colspan=\"2\">$WM3_cCLASSES_LOADED</td>
              </tr>"; 

     // Other informations
     $str .= "<tr bgcolor=\"#66ddff\"><td colspan=\"2\"><b>Other informations:</b></td></tr>";
	 $str .= "<tr bgcolor=\"#ffffff\"><td><b>Stage (filtered value)</b></td> <td><b>".$this->getStage()."</b></td></tr>";
     $str .= "<tr bgcolor=\"#dddddd\"><td><b>WM3_cQUERY_AFFECTED</b></td> <td><b>$WM3_cQUERY_AFFECTED</b></td></tr>";
     $str .= "<tr bgcolor=\"#ffffff\"><td><b>Page generated time:</b></td><td><b>".wm3Util::delayTime()."</b></td></tr>";
     $str .= "</table><br>";
     echo $str;
    }

  }

?>
